
--
-- Helper functions
--

local function get_sign(i)
	if i == 0 then
		return 0
	else
		return i/math.abs(i)
	end
end

local function get_velocity(vx, vy, vz, yaw)
	local x = math.cos(yaw)*vx+math.cos(math.pi/2+yaw)*vz
	local z = math.sin(yaw)*vx+math.sin(math.pi/2+yaw)*vz
	return {x=x, y=vy, z=z}
end

local function get_v(v)
	return math.sqrt(v.x^2+v.z^2)
end
--
--BOOM!
--
local destroy = function(pos)
	local nodename = minetest.env:get_node(pos).name
	if nodename ~= "air" then
		minetest.env:remove_node(pos)
		nodeupdate(pos)
		if minetest.registered_nodes[nodename].groups.flammable ~= nil then
			minetest.env:set_node(pos, {name="fire:basic_flame"})
			return
		end
		if math.random(1,3) == 3 then
			return
		end
		local drop = minetest.get_node_drops(nodename, "")
		for _,item in ipairs(drop) do
			if type(item) == "string" then
				local obj = minetest.env:add_item(pos, item)
				if obj == nil then
					return
				end
				obj:get_luaentity().collect = true
				obj:setacceleration({x=0, y=-10, z=0})
				obj:setvelocity({x=math.random(0,6)-3, y=10, z=math.random(0,6)-3})
			else
				for i=1,item:get_count() do
					local obj = minetest.env:add_item(pos, item:get_name())
					if obj == nil then
						return
					end
					obj:get_luaentity().collect = true
					obj:setacceleration({x=0, y=-10, z=0})
					obj:setvelocity({x=math.random(0,6)-3, y=10, z=math.random(0,6)-3})
				end
			end
		end
	end
end

boom = function(pos)
	minetest.env:set_node(pos, {name="helicopter:boom"})
		minetest.after(0.5, function(pos)
			minetest.env:remove_node(pos)
		end, {x=pos.x, y=pos.y, z=pos.z})
	minetest.sound_play("tnt_explode", {pos=pos, gain=1.5, max_hear_distance=2*64})
	local objects = minetest.env:get_objects_inside_radius(pos, 7)
	for _,obj in ipairs(objects) do
		if obj:is_player() or (obj:get_luaentity() and obj:get_luaentity().name ~= "__builtin:item") then
			local obj_p = obj:getpos()
			local vec = {x=obj_p.x-pos.x, y=obj_p.y-pos.y, z=obj_p.z-pos.z}
			local dist = (vec.x^2+vec.y^2+vec.z^2)^0.5
			local damage = (80*0.5^dist)*2
			obj:punch(obj, 1.0, {
				full_punch_interval=1.0,
				damage_groups={fleshy=damage},
			}, vec)
		end
	end
	
	for dx=-2,2 do
		for dz=-2,2 do
			for dy=2,-2,-1 do
				pos.x = pos.x+dx
				pos.y = pos.y+dy
				pos.z = pos.z+dz
				
				local node =  minetest.env:get_node(pos)
				if node.name == "tnt:tnt" or node.name == "tnt:tnt_burning" then
					minetest.env:set_node(pos, {name="tnt:tnt_burning"})
					boom({x=pos.x, y=pos.y, z=pos.z}, 0)
				elseif node.name == "fire:basic_flame" or string.find(node.name, "default:water_") or string.find(node.name, "default:lava_") or node.name == "tnt:boom" then
					
				else
					if math.abs(dx)<2 and math.abs(dy)<2 and math.abs(dz)<2 then
						destroy(pos)
					else
						if math.random(1,5) <= 4 then
							destroy(pos)
						end
					end
				end
				
				pos.x = pos.x-dx
				pos.y = pos.y-dy
				pos.z = pos.z-dz
			end
		end
	end
	
	minetest.add_particlespawner(
		100, --amount
		0.1, --time
		{x=pos.x-3, y=pos.y-3, z=pos.z-3}, --minpos
		{x=pos.x+3, y=pos.y+3, z=pos.z+3}, --maxpos
		{x=-0, y=-0, z=-0}, --minvel
		{x=0, y=0, z=0}, --maxvel
		{x=-0.5,y=5,z=-0.5}, --minacc
		{x=0.5,y=5,z=0.5}, --maxacc
		0.1, --minexptime
		1, --maxexptime
		8, --minsize
		15, --maxsize
		false, --collisiondetection
		"tnt_smoke.png" --texture
	)
end
minetest.register_node("helicopter:boom", {
	drawtype = "plantlike",
	tiles = {"tnt_boom.png"},
	light_source = LIGHT_MAX,
	walkable = false,
	drop = "",
	groups = {dig_immediate=3},
})


--
-- Rocket 
--

rocket_velocity=19

local function launch_rocket(player,left)
	local playerpos=player:getpos()
	local obj
	if left then
		obj=minetest.env:add_entity({x=playerpos.x+1*math.sin(-player:get_look_yaw()),y=playerpos.y+1,z=playerpos.z+1*math.cos(-player:get_look_yaw())}, "helicopter:rocket")
	else
		obj=minetest.env:add_entity({x=playerpos.x-1*math.sin(-player:get_look_yaw()),y=playerpos.y+1,z=playerpos.z-1*math.cos(-player:get_look_yaw())}, "helicopter:rocket")
	end
	local dir=player:get_look_dir()
	obj:setvelocity({x=dir.x*rocket_velocity, y=dir.y*rocket_velocity, z=dir.z*rocket_velocity})
	obj:setacceleration({x=dir.x*1, y=dir.y*1, z=dir.z*1})
	obj:setyaw(player:get_look_yaw())
end

rocket={
	physical = false,
	visual = "mesh",
	mesh = "rocket.x",
	timer=0,
	textures = {"rocket.png"},
	lastpos={},
	collisionbox = {0,0,0,0,0,0},
}

rocket.on_step = function(self, dtime)
	self.timer=self.timer+dtime
	local pos = self.object:getpos()
	local node = minetest.env:get_node(pos)
	if self.timer>5 then
		self.object:remove()
	end
	
	if self.lastpos.x~=nil then 
		if node.name ~= "air" then
			boom(self.lastpos)
			self.object:remove()
		end
	end
	self.lastpos={x=pos.x, y=pos.y, z=pos.z} 
end

--
-- Heli entity
--

local heli = {
	physical = true,
	collisionbox = {-1,-0.6,-1, 1,0.3,1},
	visual = "mesh",
	mesh = "heli.x",
	textures = {"heli.png"},
	driver = nil,
	left = true,
	timer=0,
	vx = 0,--Velo. for/back-ward
	vy = 0,--Velo. up/down
	vz = 0--Velo. side
}

function heli:on_rightclick(clicker)
	if not clicker or not clicker:is_player() then
		return
	end
	if self.driver and clicker == self.driver then
		self.driver = nil
		clicker:set_detach()
		self.object:set_animation({x=0,y=1},0, 0)
	elseif not self.driver then
		self.object:set_animation({x=0,y=10},10, 0)
		self.driver = clicker
		clicker:set_attach(self.object, "", {x=0,y=2,z=0}, {x=0,y=0,z=0})
		self.object:setyaw(clicker:get_look_yaw())
	end
end

function heli:on_activate(staticdata, dtime_s)
	self.object:set_armor_groups({immortal=1})
	if staticdata then
		self.v = tonumber(staticdata)
	end
end

function heli:get_staticdata()
	return tostring(v)
end

function heli:on_punch(puncher, time_from_last_punch, tool_capabilities, direction)
	self.object:remove()
	if puncher and puncher:is_player() then
		puncher:get_inventory():add_item("main", "helicopter:heli")
	end
end

function heli:on_step(dtime)
	self.timer=self.timer+dtime
	--self.v = get_v(self.object:getvelocity())*get_sign(self.v)
	if self.driver then
		local ctrl = self.driver:get_player_control()
		if ctrl.LMB and self.timer>1 then
			if self.left then
				self.left = false
			else 
				self.left = true
			end
			launch_rocket(self.driver,self.left)
			self.timer = 0
		end
		--Forward/backward
		if ctrl.up then
			self.vx = self.vx+0.1
		end
		if ctrl.down then
			self.vx = self.vx-0.08
		end
		--Left/right
		if ctrl.left then
			self.vz = self.vz+0.1
		end
		if ctrl.right then
			self.vz = self.vz-0.1
		end
		--up/down
		if ctrl.jump then
			if self.vy<1.5 then
				self.vy = self.vy+0.2
			end
		end
		if ctrl.sneak then
			if self.vy>-1.5 then
				self.vy = self.vy-0.2
			end
		end
		--
		self.object:setyaw(self.driver:get_look_yaw())
	end
	--Decelerating
	local sx=get_sign(self.vx)
	self.vx = self.vx - 0.02*sx
	
	local sz=get_sign(self.vz)
	self.vz = self.vz - 0.02*sz
	
	local sy=get_sign(self.vy)
	self.vy = self.vy-0.01*sy
	
	--Stop
	if sx ~= get_sign(self.vx) then
		self.vx = 0
	end
	
	if sz ~= get_sign(self.vz) then
		self.vz = 0
	end
	
	if sy ~= get_sign(self.vy) then
		self.vy = 0
	end
	
	--Speed limit
	if math.abs(self.vx) > 4.5 then
		self.vx = 4.5*get_sign(self.vx)
	end
	if math.abs(self.vz) > 4.5 then
		self.vz = 4.5*get_sign(self.vz)
	end
	--Set speed to entity
	self.object:setvelocity(get_velocity(self.vx, self.vy, self.vz, self.object:getyaw()))
end

--
--Registration
--

minetest.register_entity("helicopter:heli", heli)
minetest.register_entity("helicopter:rocket", rocket)

minetest.register_craftitem("helicopter:heli", {
	description = "Heli",
	inventory_image = "heli_inv.png",
	wield_image = "heli_inv.png",
	wield_scale = {x=1, y=1, z=1},
	liquids_pointable = true,
	
	on_place = function(itemstack, placer, pointed_thing)
		if pointed_thing.type ~= "node" then
			return
		end
		pointed_thing.under.y = pointed_thing.under.y+1
		minetest.env:add_entity(pointed_thing.under, "helicopter:heli")
		itemstack:take_item()
		return itemstack
	end,
})

minetest.register_craft({
	output = "helicopter:heli",
	recipe = {
		{"group:wood", "group:wood", ""},
		{"group:wood", "", "group:wood"},
		{"group:wood", "group:wood", "group:wood"},
	},
})
